var root = document.getElementById("tabs");
var links = ['left', 'right'];

function elementToTabId(element) {
    return parseInt(element.getAttribute('id').substring(4));
}

function tabIdToElement(tabId) {
    return document.getElementById('tab-' + tabId);
}

function tabStripEvent(event) {
    /* An event on the tab strip */
    let name = event.target.className;
    let tabId = elementToTabId(event.target.parentElement);
    chrome.tabs.get(tabId, function(tab) {
        if (name == 'left') {
            let index = tab.index - 1;
            chrome.tabs.move([tab.id], {index});
        } else if (name == 'right') {
            let index = tab.index + 1;
            chrome.tabs.move([tab.id], {index});
        }
    })
    event.preventDefault();
}

function createTabStripEntry(tab) {
    console.log('Creating tab:', tab.id, tab.url);
    
    var tabElement = document.createElement('div');
    tabElement.setAttribute('id', 'tab-' + tab.id);
    
    var urlElement = document.createElement('span');
    urlElement.innerText = tab.url;
    tabElement.appendChild(urlElement);
    
    for (let link of links) {
        let linkElement = document.createElement('a');
        linkElement.setAttribute('href', '#');
        linkElement.className = link;
        linkElement.innerText = link;
        linkElement.addEventListener('click', tabStripEvent);
        tabElement.appendChild(linkElement);
    }
    root.appendChild(tabElement);
}

function onMoved(tabId, info) {
    console.log('onMoved:', tabId, info);
    var tab = tabIdToElement(tabId);
    if (info.fromIndex > info.toIndex) {
        tabs.insertBefore(
            tabs.childNodes[info.fromIndex],
            tabs.childNodes[info.toIndex]
        );
    } else if (info.toIndex > info.fromIndex) {
        tabs.insertBefore(
            tabs.childNodes[info.toIndex],
            tabs.childNodes[info.fromIndex]
        );        
    }
}

function onUpdated(tabId, info) {
    console.log('onUpdated:', tabId, info);
    var tab = tabIdToElement(tabId);
    if (info.hasOwnProperty('url')) {
        tabElement.childNodes[0].innerText = info.url;
    }
}

function onCreated(tab) {
    console.log('onCreated:', tab);
    createTabStripEntry(tab);
}

function onRemoved(tabId) {
    console.log('onRemoved:', tabId);
    tabIdToElement(tabId).remove();
}

browser.tabs.query({currentWindow: true}, function(tabs) {
    for (let tab of tabs) {
        createTabStripEntry(tab);
    }
});

browser.tabs.onCreated.addListener(onCreated);
browser.tabs.onMoved.addListener(onMoved);
browser.tabs.onRemoved.addListener(onRemoved);
browser.tabs.onUpdated.addListener(onUpdated);

